@@ -37,7 +37,7 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: depmod.c 1.23 Sat, 23 Sep 2000 14:13:54 +1100 kaos $"
+#ident "$Id: depmod.c 1.25 Tue, 10 Oct 2000 12:16:02 +1100 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -84,6 +84,23 @@ struct pci_device_id {
 	unsigned long driver_data;		/* Data private to the driver */
 };
 
+/* Extracted from 2.4.0-test10-pre1 /include/linux/isapnp.h */
+struct isapnp_device_id {
+	unsigned short card_vendor, card_device;
+	unsigned short vendor, function;
+	unsigned long driver_data;	/* data private to the driver */
+};
+
+/* Extracted from 2.4.0-test10-pre1 /include/linux/isapnp.h */
+#define ISAPNP_CARD_DEVS	8
+struct isapnp_card_id {
+	unsigned long driver_data;	/* data private to the driver */
+	unsigned short card_vendor, card_device;
+	struct {
+		unsigned short vendor, function;
+	} devs[ISAPNP_CARD_DEVS];	/* logical devices */
+};
+
 typedef struct MODULE {
 	char *name;
 	int resolved;
@@ -91,8 +108,12 @@ typedef struct MODULE {
 		SYMBOL **symtab;
 		int n_syms;
 	} defsym, undefs;
-	struct pci_device_id *pci;
-	int n_pci;
+	struct pci_device_id *pci_device;
+	int n_pci_device;
+	struct isapnp_device_id *isapnp_device;
+	int n_isapnp_device;
+	struct isapnp_card_id *isapnp_card;
+	int n_isapnp_card;
 } MODULE;
 
 typedef enum SYM_STATUS {
@@ -275,25 +296,107 @@ static int in_range(const struct obj_file *f, unsigned long m_size,
 	return(start >= f->baseaddr && (start + length - 1 - f->baseaddr) <= m_size);
 }
 
+static const char has_kernel_changed[] =
+	"\n\tIt is likely that the kernel structure has changed, if so then"
+	"\n\tyou probably need a new version of modutils to handle this kernel."
+	"\n\tCheck linux/Documentation/Changes.";
+
 /*
  *	Extract any pci_device_table from the module.
  *	Note: assumes same machine and arch for depmod and module.
  */
 static void extract_pci_device_id(struct obj_file *f, void *image, unsigned long m_size, MODULE *mod)
 {
-	struct pci_device_id pci;
+	struct pci_device_id pci_device;
 	ElfW(Addr) ref_pci;
-	ElfW(Addr) ref_ref_pci = obj_symbol_final_value(f, obj_find_symbol(f, "__module_pci_device_table"));
+	ElfW(Addr) ref_ref_pci;
+	unsigned long pci_device_size;
+	ref_pci = obj_symbol_final_value(f, obj_find_symbol(f, "__module_pci_device_size"));
+	/* Kernels before 2.4.0-test10-pre1 do not have __module_pci_device_size */
+	if (ref_pci) {
+		if (!in_range(f, m_size, ref_pci, sizeof(pci_device_size)))
+			return;
+		memcpy(&pci_device_size, (char *)image + ref_pci - f->baseaddr, sizeof(pci_device_size));
+	}
+	else
+		pci_device_size = sizeof(pci_device);
+	if (pci_device_size != sizeof(pci_device)) {
+		error("Unexpected value (%ld) for pci_device_size%s", pci_device_size, has_kernel_changed);
+		exit(-1);
+	}
+	ref_ref_pci = obj_symbol_final_value(f, obj_find_symbol(f, "__module_pci_device_table"));
 	if (!in_range(f, m_size, ref_ref_pci, sizeof(ref_pci)))
 		return;
 	memcpy(&ref_pci, (char *)image + ref_ref_pci - f->baseaddr, sizeof(ref_pci));
-	while (in_range(f, m_size, ref_pci, sizeof(pci))) {
-		memcpy(&pci, (char *)image + ref_pci - f->baseaddr, sizeof(pci));
-		ref_pci += sizeof(pci);
-		if (!pci.vendor)
+	while (in_range(f, m_size, ref_pci, sizeof(pci_device))) {
+		memcpy(&pci_device, (char *)image + ref_pci - f->baseaddr, sizeof(pci_device));
+		ref_pci += sizeof(pci_device);
+		if (!pci_device.vendor)
 			break;
-		mod->pci = xrealloc(mod->pci, ++(mod->n_pci)*sizeof(*(mod->pci)));
-		mod->pci[mod->n_pci-1] = pci;
+		mod->pci_device = xrealloc(mod->pci_device, ++(mod->n_pci_device)*sizeof(*(mod->pci_device)));
+		mod->pci_device[mod->n_pci_device-1] = pci_device;
+	}
+}
+
+/*
+ *	Extract any isapnp_device_table from the module.
+ *	Note: assumes same machine and arch for depmod and module.
+ */
+static void extract_isapnp_device_id(struct obj_file *f, void *image, unsigned long m_size, MODULE *mod)
+{
+	struct isapnp_device_id isapnp_device;
+	ElfW(Addr) ref_isapnp, ref_ref_isapnp;
+	unsigned long isapnp_device_size;
+	ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_device_size"));
+	if (!in_range(f, m_size, ref_isapnp, sizeof(isapnp_device_size)))
+		return;
+	memcpy(&isapnp_device_size, (char *)image + ref_isapnp - f->baseaddr, sizeof(isapnp_device_size));
+	if (isapnp_device_size != sizeof(isapnp_device)) {
+		error("Unexpected value (%ld) for isapnp_device_size%s", isapnp_device_size, has_kernel_changed);
+		exit(-1);
+	}
+	ref_ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_device_table"));
+	if (!in_range(f, m_size, ref_ref_isapnp, sizeof(ref_isapnp)))
+		return;
+	memcpy(&ref_isapnp, (char *)image + ref_ref_isapnp - f->baseaddr, sizeof(ref_isapnp));
+	while (in_range(f, m_size, ref_isapnp, sizeof(isapnp_device))) {
+		memcpy(&isapnp_device, (char *)image + ref_isapnp - f->baseaddr, sizeof(isapnp_device));
+		ref_isapnp += sizeof(isapnp_device);
+		if (!isapnp_device.card_vendor)
+			break;
+		mod->isapnp_device = xrealloc(mod->isapnp_device, ++(mod->n_isapnp_device)*sizeof(*(mod->isapnp_device)));
+		mod->isapnp_device[mod->n_isapnp_device-1] = isapnp_device;
+	}
+}
+
+/*
+ *	Extract any isapnp_card_table from the module.
+ *	Note: assumes same machine and arch for depmod and module.
+ */
+static void extract_isapnp_card_id(struct obj_file *f, void *image, unsigned long m_size, MODULE *mod)
+{
+	struct isapnp_card_id isapnp_card;
+	ElfW(Addr) ref_isapnp, ref_ref_isapnp;
+	unsigned long isapnp_card_size;
+	ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_card_size"));
+	if (!in_range(f, m_size, ref_isapnp, sizeof(isapnp_card_size)))
+		return;
+	memcpy(&isapnp_card_size, (char *)image + ref_isapnp - f->baseaddr, sizeof(isapnp_card_size));
+	if (isapnp_card_size != sizeof(isapnp_card)) {
+		error("Unexpected value (%ld) for isapnp_card_size%s", isapnp_card_size, has_kernel_changed);
+		exit(-1);
+	}
+	ref_ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_card_table"));
+	if (!in_range(f, m_size, ref_ref_isapnp, sizeof(ref_isapnp)))
+		return;
+	memcpy(&ref_isapnp, (char *)image + ref_ref_isapnp - f->baseaddr, sizeof(ref_isapnp));
+	while (in_range(f, m_size, ref_isapnp, sizeof(isapnp_card))) {
+		memcpy(&isapnp_card, (char *)image + ref_isapnp - f->baseaddr, sizeof(isapnp_card));
+		ref_isapnp += sizeof(isapnp_card);
+		if (!isapnp_card.card_vendor)
+			break;
+		mod->isapnp_card = xrealloc(mod->isapnp_card, ++(mod->n_isapnp_card)*sizeof(*(mod->isapnp_card)));
+		mod->isapnp_card[mod->n_isapnp_card-1] = isapnp_card;
 	}
 }
 
@@ -470,6 +573,8 @@ static int loadobj(const char *objname, int ignore_suffix)
 	image = xmalloc(m_size);
 	obj_create_image(f, image);
 	extract_pci_device_id(f, image, m_size, mod);
+	extract_isapnp_device_id(f, image, m_size, mod);
+	extract_isapnp_card_id(f, image, m_size, mod);
 	free(image);
 
 	obj_free(f);
@@ -536,12 +641,14 @@ static int addksyms(char *file_syms)
 			return -1;
 		}
 
-		if (!strtok(line, " \t\n")) {
+		p = strtok(line, " \t\n");
+		if (!p) {
 			fclose(fp);
 			error("Illegal format of %s", file_syms);
 			return -1;
 		}
-		p = strtok(NULL, " \t\n");
+		if (!isspace(*line))	/* Adressless symbol? */
+			p = strtok(NULL, " \t\n");
 		/* The second word is either the symbol name or a type */
 		if (p && strlen(p) == 1) { /* System.map */
 			is_mapfile = 1;
@@ -560,8 +667,9 @@ static int addksyms(char *file_syms)
 		while (fgets(line, sizeof(line), fp)) {
 			if (!is_mapfile && strchr(line, '['))
 				continue;
-			strtok(line, " \t\n"); /* Skip first word */
-			p = strtok(NULL, " \n");
+			p = strtok(line, " \t\n"); /* Skip first word */
+			if (!isspace(*line))	/* Adressless symbol? */
+				p = strtok(NULL, " \t\n");
 			if (is_mapfile) {
 				if (*p != '?')
 					continue;
@@ -596,11 +704,16 @@ static int addksyms(char *file_syms)
  *	Format the dependancy list of a module into a simple makefile.
  *	Print the dependancies in the depfile (or stdout if depfile is NULL).
  *	Print the pcimap in the pcimapfile (or stdout if pcimapfile is NULL).
+ *	Print the isapnpmap in the isapnpmapfile (or stdout if isapnpmapfile is NULL).
  */
-static void prtdepend(const char *pdepfile, const char *ppcimapfile, char *base_dir)
+static void prtdepend(char *base_dir,
+		      const char *pdepfile,
+		      const char *ppcimapfile,
+		      const char *pisapnpmapfile)
 {
 	FILE *dep = stdout;
 	FILE *pcimap = stdout;
+	FILE *isapnpmap = stdout;
 	MODULE **tbdep;
 	MODULE *ptmod;
 	int i;
@@ -620,6 +733,13 @@ static void prtdepend(const char *pdepfile, const char *ppcimapfile, char *base_
 		}
 	}
 
+	if (pisapnpmapfile != NULL) {
+		if ((isapnpmap = fopen(pisapnpmapfile, "w")) == NULL) {
+			error("Can't open %s for writing", pisapnpmapfile);
+			exit(-1);
+		}
+	}
+
 	skipchars = strlen(base_dir);
 	tbdep = (MODULE **)alloca(sizeof(MODULE) * n_modules);
 
@@ -677,9 +797,9 @@ static void prtdepend(const char *pdepfile, const char *ppcimapfile, char *base_
 	fprintf(pcimap, "# module             vendor     device     subvendor  subdevice  class      class_mask driver_data\n");
 	for (i = 0; i < n_modules; i++, ptmod++) {
 		int j, l;
-		struct pci_device_id *pci = ptmod->pci;
+		struct pci_device_id *pci_device = ptmod->pci_device;
 		char *name, *shortname;
-		if (!ptmod->n_pci)
+		if (!ptmod->n_pci_device)
 			continue;
 		name = xstrdup(ptmod->name);
 		shortname = strrchr(name, '/');
@@ -700,16 +820,71 @@ static void prtdepend(const char *pdepfile, const char *ppcimapfile, char *base_
 				l -= 4;
 			}
 		}
-		for (j = 0; j < ptmod->n_pci; j++, pci++) {
+		for (j = 0; j < ptmod->n_pci_device; j++, pci_device++) {
 			fprintf(pcimap, "%-20s 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*lx\n",
 				shortname,
-				2*sizeof(pci->vendor), pci->vendor,
-				2*sizeof(pci->device), pci->device,
-				2*sizeof(pci->subvendor), pci->subvendor,
-				2*sizeof(pci->subdevice), pci->subdevice,
-				2*sizeof(pci->class), pci->class,
-				2*sizeof(pci->class_mask), pci->class_mask,
-				2*sizeof(pci->driver_data), pci->driver_data);
+				2*sizeof(pci_device->vendor), pci_device->vendor,
+				2*sizeof(pci_device->device), pci_device->device,
+				2*sizeof(pci_device->subvendor), pci_device->subvendor,
+				2*sizeof(pci_device->subdevice), pci_device->subdevice,
+				2*sizeof(pci_device->class), pci_device->class,
+				2*sizeof(pci_device->class_mask), pci_device->class_mask,
+				2*sizeof(pci_device->driver_data), pci_device->driver_data);
+		}
+		free(name);
+	}
+
+	ptmod = modules;
+	fprintf(isapnpmap, "# module             cardvendor carddevice driver_data vendor     function   ...\n");
+	for (i = 0; i < n_modules; i++, ptmod++) {
+		int j, l;
+		struct isapnp_device_id *isapnp_device = ptmod->isapnp_device;
+		struct isapnp_card_id *isapnp_card = ptmod->isapnp_card;
+		char *name, *shortname;
+		if (!ptmod->n_isapnp_device && !ptmod->n_isapnp_card)
+			continue;
+		name = xstrdup(ptmod->name);
+		shortname = strrchr(name, '/');
+		if (!shortname)
+			shortname = name;	/* should never happen */
+		else {
+			++shortname;
+			l = strlen(shortname);
+			if (l > 3 && strcmp(shortname+l-3, ".gz") == 0) {
+				*(shortname+l-3) = '\0';
+				l -= 3;
+			}
+			if (l > 2 && strcmp(shortname+l-2, ".o") == 0) {
+				*(shortname+l-2) = '\0';
+				l -= 2;
+			} else if (l > 4 && strcmp(shortname+l-4, ".mod") == 0) {
+				*(shortname+l-4) = '\0';
+				l -= 4;
+			}
+		}
+		for (j = 0; j < ptmod->n_isapnp_device; j++, isapnp_device++) {
+			fprintf(isapnpmap, "%-20s 0x%0*x     0x%0*x     0x%0*lx  0x%0*x     0x%0*x\n",
+				shortname,
+				2*sizeof(isapnp_device->card_vendor), isapnp_device->card_vendor,
+				2*sizeof(isapnp_device->card_device), isapnp_device->card_device,
+				2*sizeof(isapnp_device->driver_data), isapnp_device->driver_data,
+				2*sizeof(isapnp_device->vendor), isapnp_device->vendor,
+				2*sizeof(isapnp_device->function), isapnp_device->function);
+		}
+		for (j = 0; j < ptmod->n_isapnp_card; j++, isapnp_card++) {
+			fprintf(isapnpmap, "%-20s 0x%0*x     0x%0*x     0x%0*lx ",
+				shortname,
+				2*sizeof(isapnp_card->card_vendor), isapnp_card->card_vendor,
+				2*sizeof(isapnp_card->card_device), isapnp_card->card_device,
+				2*sizeof(isapnp_card->driver_data), isapnp_card->driver_data);
+			for (l = 0; l < ISAPNP_CARD_DEVS; ++l) {
+				if (isapnp_card->devs[l].vendor) {
+					fprintf(isapnpmap, " 0x%0*x     0x%0*x    ",
+						2*sizeof(isapnp_card->devs[l].vendor), isapnp_card->devs[l].vendor,
+						2*sizeof(isapnp_card->devs[l].function), isapnp_card->devs[l].function);
+				}
+			}
+			fprintf(isapnpmap, "\n");
 		}
 		free(name);
 	}
@@ -717,6 +892,8 @@ static void prtdepend(const char *pdepfile, const char *ppcimapfile, char *base_
 	/* Close depfile last, critical timestamp */
 	if (ppcimapfile != NULL)
 		fclose(pcimap);
+	if (pisapnpmapfile != NULL)
+		fclose(isapnpmap);
 	if (pdepfile != NULL)
 		fclose(dep);
 }
@@ -907,7 +1084,10 @@ int DEPMOD_MAIN(int argc, char **argv)
 			for (i = 0; g && i < g->pathc; i++)
 				loadobj(g->pathv[i], ignore_suffix);
 			resolve();
-			prtdepend(nflag ? NULL : depfile, nflag ? NULL : pcimapfile, base_dir);
+			prtdepend(base_dir,
+				  nflag ? NULL : depfile,
+				  nflag ? NULL : pcimapfile,
+				  nflag ? NULL : isapnpmapfile);
 		}
 	} else {
 		/* not stdmode */
@@ -915,7 +1095,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 			for (; argc > 0 && ret != -1; ++argv, --argc)
 				loadobj(*argv, ignore_suffix);
 			resolve();
-			prtdepend(NULL, NULL, base_dir);
+			prtdepend(base_dir, NULL, NULL, NULL);
 		}
 	}
 
