@@ -21,7 +21,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_load.c 1.7 Fri, 08 Sep 2000 16:46:27 +1100 kaos $"
+#ident "$Id: obj_load.c 1.9 Tue, 10 Oct 2000 22:05:46 +1100 kaos $"
 
 #include <alloca.h>
 #include <string.h>
@@ -61,7 +61,7 @@ obj_load (int fp, Elf32_Half e_type, const char *filename)
       || f->header.e_ident[EI_MAG2] != ELFMAG2
       || f->header.e_ident[EI_MAG3] != ELFMAG3)
     {
-      error("not an ELF file");
+      error("%s is not an ELF file", filename);
       return NULL;
     }
   if (f->header.e_ident[EI_CLASS] != ELFCLASSM
@@ -151,11 +151,13 @@ obj_load (int fp, Elf32_Half e_type, const char *filename)
 
 #if SHT_RELM == SHT_REL
 	case SHT_RELA:
-	  error("RELA relocations not supported on this architecture");
+	  if (sec->header.sh_size)
+	    error("RELA relocations not supported on this architecture");
 	  return NULL;
 #else
 	case SHT_REL:
-	  error("REL relocations not supported on this architecture");
+	  if (sec->header.sh_size)
+	    error("REL relocations not supported on this architecture");
 	  return NULL;
 #endif
 
