@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_s390.c 1.2 Mon, 10 Jul 2000 22:23:47 -0700 kaos $"
+#ident "$Id: obj_s390.c 1.3 Wed, 23 Aug 2000 10:23:26 +1000 kaos $"
 
 #include <string.h>
 #include <assert.h>
@@ -226,3 +226,9 @@ arch_init_module (struct obj_file *f, struct module *m)
 {
   return 1;
 }
+
+int
+arch_archdata (struct obj_file *fin, struct obj_section *sec)
+{
+  return 0;
+}
