@@ -211,6 +211,7 @@ char *prune[] =
 {
 	"modules.dep",
 	"modules.pcimap",
+	"modules.isapnpmap",
 	"System.map",
 	".config",
 	"build",		/* symlink to source tree */
@@ -218,5 +219,6 @@ char *prune[] =
 	"vmlinuz",
 	"bzImage",
 	"zImage",
+	".rhkmvtag",		/* wish RedHat had told me before they did this */
 	NULL			/* marks the end of the list! */
 };
