@@ -101,6 +101,7 @@ static int n_aliases;
 char *insmod_opt = NULL;
 char *depfile = NULL;
 char *pcimapfile = NULL;
+char *isapnpmapfile = NULL;
 char *config_file = NULL;	/* Which file was actually used */
 time_t config_mtime;
 int root_check_off = CONFIG_ROOT_CHECK_OFF;	/* Default is modules must be owned by root */
@@ -127,6 +128,7 @@ void verbose(const char *ctl,...)
  */
 static time_t dep_mtime;
 static time_t pcimap_mtime;
+static time_t isapnpmap_mtime;
 static int check_update (const char *file, const struct stat *sb)
 {
 	int len = strlen(file);
@@ -134,7 +136,8 @@ static int check_update (const char *file, const struct stat *sb)
 	if (!S_ISREG(sb->st_mode))
 		return 0;
 	if (sb->st_mtime <= dep_mtime &&
-	    sb->st_mtime <= pcimap_mtime)
+	    sb->st_mtime <= pcimap_mtime &&
+	    sb->st_mtime <= isapnpmap_mtime)
 		return 0;
 
 	if (len > 2 && !strcmp(file + len - 2, ".o"))
@@ -150,7 +153,7 @@ static int check_update (const char *file, const struct stat *sb)
 
 static int need_update (const char *force_ver, const char *base_dir)
 {
-	struct stat statdep, statpcimap, tmp;
+	struct stat statdep, statpcimap, statisapnpmap, tmp;
 	char dep[PATH_MAX];
 	uname (&uts_info);
 	if (!force_ver)
@@ -161,7 +164,8 @@ static int need_update (const char *force_ver, const char *base_dir)
 		return 1;
 
 	if (stat (depfile, &statdep) ||
-	    stat (pcimapfile, &statpcimap))
+	    stat (pcimapfile, &statpcimap) ||
+	    stat (isapnpmapfile, &statisapnpmap))
 		/* No dependency file yet, so we need to build it. */
 		return 1;
 
@@ -170,12 +174,14 @@ static int need_update (const char *force_ver, const char *base_dir)
 		return 1;
 
 	if (tmp.st_mtime > statdep.st_mtime ||
-	    tmp.st_mtime > statpcimap.st_mtime)
+	    tmp.st_mtime > statpcimap.st_mtime ||
+	    tmp.st_mtime > statisapnpmap.st_mtime)
 		/* Config file is newer. */
 		return 1;
 
 	dep_mtime = statdep.st_mtime;
 	pcimap_mtime = statpcimap.st_mtime;
+	isapnpmap_mtime = statisapnpmap.st_mtime;
 	sprintf (dep, "%s/lib/modules/%s", base_dir, force_ver);
 	return xftw (dep, check_update);
 }
@@ -412,6 +418,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	char *type;
 	char depfile_tmp[PATH_MAX];
 	char pcimapfile_tmp[PATH_MAX];
+	char isapnpmapfile_tmp[PATH_MAX];
 	char **glb;
 	char old_name[] = "/etc/conf.modules";
 	int conf_file_specified = 0;
@@ -567,6 +574,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 
 	depfile_tmp[0] = '\0';
 	pcimapfile_tmp[0] = '\0';
+	isapnpmapfile_tmp[0] = '\0';
 
 	/* Only read the default entries on the first file */
 	if (depth == 0) {
@@ -699,6 +707,11 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			strncpy(pcimapfile_tmp, envpath, sizeof(pcimapfile_tmp));
 		}
 
+		/* ISAPNPMAPPATH is intended for testing only */
+		if ((envpath = getenv("ISAPNPMAPPATH")) != NULL) {
+			strncpy(isapnpmapfile_tmp, envpath, sizeof(isapnpmapfile_tmp));
+		}
+
 	}	/* End of depth == 0 */
 
 	if (conf_file ||
@@ -1176,6 +1189,24 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			one_err = 0;
 		}
 
+		/*
+		 * isapnpmapfile=
+		 */
+		else if (assgn && (strcmp(parm, "isapnpmapfile") == 0) &&
+			 (isapnpmapfile_tmp[0] == '\0')) {
+			/*
+			 * The default value for the isapnpmapfile parameter is:
+			 *
+			 * isapnpmapfile=/lib/modules/`uname -r`/modules.isapnpmap
+			 *
+			 * If the config file exist but doesn't have a isapnpmapfile
+			 * specification, the default is used since modprobe
+			 * can't work without one.
+			 */
+			strcpy(isapnpmapfile_tmp, arg);
+			one_err = 0;
+		}
+
 		/*
 		 * keep
 		 */
@@ -1354,6 +1385,36 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			pcimapfile = g.pathv[0];
 	}
 
+	/*
+	 * Make sure we have a definition of "isapnpmapfile"
+	 */
+	if (isapnpmapfile_tmp[0] == '\0') {
+		/*
+		 * Specification: config file / no isapnpmapfile parameter
+		 * The default value for the isapnpmapfile parameter is:
+		 *
+		 * isapnpmapfile=/lib/modules/`uname -r`/modules.isapnpmap
+		 *
+		 * If the config file exists but lack a isapnpmapfile
+		 * specification, the default value is used since
+		 * the system can't work without one.
+		 */
+		sprintf(isapnpmapfile_tmp, "%s/lib/modules/%s/modules.isapnpmap",
+			base_dir, version);
+		isapnpmapfile = xstrdup(isapnpmapfile_tmp);
+	} else { /* isapnpmapfile defined in modules.conf */
+		/*
+		 * If we have a isapnpmapfile definition in the configuration file
+		 * we must resolve any shell meta-chars in its value.
+		 */
+		if (meta_expand(isapnpmapfile_tmp, &g, base_dir, version))
+			ret = -1;
+		else if (!g.pathv || g.pathv[0] == NULL)
+			ret = -1;
+		else
+			isapnpmapfile = g.pathv[0];
+	}
+
 	return ret;
 }
 
